<?php

namespace App\Util;

use App\Exception;
use App\Util\Security as Security;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

class Util {

    private $connection;
    
    public function __construct(\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }

    public function writeLog($type, $fileName, $content, $client = null) {

        $configFile = file_get_contents(__DIR__ . "/../../environment/config.json");
        $configData = json_decode($configFile, true);
        $basePath   = $configData['basePath'];

        if($type == "requests") {
            $file = "$basePath/$fileName";
            file_put_contents($file, ($content . PHP_EOL), FILE_APPEND);
        }

        if($type == "general") {
            $folder = $basePath . '/general/';
            $folder .= date("Y") . "/" . date("m") . "/" . date("d") . "/";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            $file = $folder . 'general.txt';
            $text = date('d/m/Y H:i:s') . " - $content\r\n";

            file_put_contents($file, $text, FILE_APPEND);
        }

        if($type == "events") {
            $data   = json_decode($content, true);
            $folder = $basePath . "/";

            if(!empty($client)) {
              $folder .= $client . "/";    
            } else {
              $folder .= "undefined/";
            }

            $folder .= "/events/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
    
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
    
            $currentFile = $folder . date("H-i-s-u") . ".json";
            file_put_contents($currentFile, $content);
        }

        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }
        
        if($type == 'cancelled') {
            $folder      = $basePath . "/cancelled/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'error') {
          $folder = $basePath . "/";

          if(!empty($client)) {
            $folder .= $client . "/";    
          } else {
            $folder .= "undefined/";
          }

          $folder .= "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
          $currentFile = "$folder/$fileName";
          if(!file_exists($folder)) {
              mkdir($folder, 0775, true);
          }

          file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
    }

    public function sendDatabase($subject, $body, $params) {
        try {
            $config = [
                "host"      => "email-smtp.us-east-1.amazonaws.com",
                "username"  => "AKIAZRKH2SFLKRV4RZW5",
                "from"      => "no-reply@teknisa.com",
                "password"  => "DJ3x6Y7+cneZegVOGW6IN3I2FRxRKVZLISjO1plGz8fB/7tOjc8/GsLb2H6UbMcUOb2iPV1CbuEbzy5CuqbmWp+7nxGHktzRwbtavQ4Qyu8Y0wOd4yf0quHzoy1Bsled",
                "SMTPSecure"=> "tls",
                "port"      => 587
            ];

            $environmentFile = file_get_contents(__DIR__ . "/../../environment/environment.json");
            $environmentData = json_decode($environmentFile, true);
            
            $path = $environmentData['path'];

            $mail = new PHPMailer(true);
            
            $mail->isSMTP(); 
            $mail->Host       = $config["host"]; 
            $mail->SMTPAuth   = true; 
            $mail->Username   = $config["username"]; 
            $mail->Password   = $config["password"]; 
            $mail->SMTPSecure = $config["SMTPSecure"]; 
            $mail->Port       = $config["port"]; 
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";

            $mail->addAttachment($path);
            
            $mail->setFrom($config["from"], "Odhen Automação Comercial");

            $mail->addAddress("delivery@teknisa.com");

            $mail->isHTML(true);                                  

            $mail->Subject = "$subject";
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();
        } catch (\Exception $e) {
            var_dump("error: ".$e->getMessage());
        }
    }

    public function tofloat($num) {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
       
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        } 
    
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );
        
        $tmp2 = str_replace(".", ",", $tmp);

        if($this->getDbSchema() == 'en-us') {
            $tmp2 = str_replace(",", ".", $tmp2);
        }
        
        return $tmp2; 
    }
    
    public function request($url, $body, $header) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if (!empty(curl_error($ch)) || (stripos($response, "<html>"))) {
            $fileName = date("H-i-s") . ".txt";
            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $url,
                'method'    => 'POST',
                'params'    => $body,
                'httpCode'  => $httpCode,
                'error'     => curl_error($ch),
                'response'  => $response
            ]));

            throw new \Exception("Erro na comunicação de chaveamento.");
        }
        
        curl_close($ch);
        return $response;
    }

    public function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }
}